==> OSAkkie0.0.15.01.zip: src/memory.c <==
/* [ */

#include "bootpack.h"

#define EFLAGS_AC_BIT		0x00040000
#define CR0_CACHE_DISABLE	0x60000000

unsigned int memtest(unsigned int start, unsigned int end)
{
	char flg486 = 0;
	unsigned int eflg, cr0, i;

	/* 386A486ȍ~mF */
	eflg = io_load_eflags() | EFLAGS_AC_BIT;
	io_store_eflags(eflg);
	eflg = io_load_eflags() & EFLAGS_AC_BIT;
	if (eflg)
		flg486 = 1;
	eflg &= ~EFLAGS_AC_BIT;		// and̂
	io_store_eflags(eflg);

	if (flg486) {
		cr0 = load_cr0() | CR0_CACHE_DISABLE;	/* LbV֎~ */
		store_cr0(cr0);
	}

	i = memtest_sub(start, end);

	if (flg486) {
		cr0 = load_cr0();
		cr0 &= ~CR0_CACHE_DISABLE;	/* LbV */
		store_cr0(cr0);
	}

	return i;
}

void memman_init(struct MEMMAN *man)
{
	man->frees = 0;		/* 󂫏̌ */
	man->maxfrees = 0;	/* 󋵊ώ@p: frees̍ől */
	man->lostsize = 0;	/* JɎsvTCY */
	man->losts = 0;		/* JɎs */
	return;
}

unsigned int memman_total(struct MEMMAN *man)
/* 󂫃TCY̍v */
{
	unsigned int i, t = 0;
	for (i = 0; i < man->frees; i++)
		t += man->free[i].size;
	return t;
}

unsigned int memman_alloc(struct MEMMAN *man, unsigned int size)
/* m */
{
	unsigned int i, a;
	for (i = 0; i < man->frees; i++) {
		if (man->free[i].size >= size) {
			/* \ȍL̋󂫂𔭌 */
			a = man->free[i].addr;
			man->free[i].addr += size;
			man->free[i].size -= size;
			if (man->free[i].size == 0) {
				/* free[i]ȂȂ̂őOɋl߂ */
				man->frees--;
				for (; i < man->frees; i++)
					man->free[i] = man->free[i + 1];
			}
			return a;
		}
	}
	return 0;	/* 󂫂Ȃ */
}

int memman_free(struct MEMMAN *man, unsigned int addr, unsigned int size)
/*  */
{
	int i, j;

	/* free[]addrɕׂ邽߂ɁAǂɓ邩߂ */
	for (i = 0; i < man->frees; i++) {
		if (man->free[i].addr > addr)
			break;
	}
	/* free[i - 1].addr < addr < free[i].addr */
	if (0 < i) {
		/* p^[1-1: ꏊO */
		if (man->free[i - 1].addr + man->free[i - 1].size == addr) {
			/* ̑Ő󂫗̈ɂ܂Ƃ߂ */
			man->free[i - 1].size += size;
			if (i < man->frees) {
				/* p^[1-2: ꂽꏊ */
				if (addr + size == man->free[i].addr) {
					/* ̗̈܂Ƃ߂ */
					man->free[i - 1].size += man->free[i].size;
					/* man->free[i]̍폜:: Oɋl߂ */
					man->frees--;
					for (; i < man->frees; i++)
						man->free[i] = man->free[i + 1];
				}
			}
			return 0;	/* E}[ */
		}
	}
	if (i < man->frees) {
		/* p^[2: ꏊ낪 */
		if (addr + size == man->free[i].addr) {
			/* ̗̈ɂ܂Ƃ߂ */
			man->free[i].addr = addr;
			man->free[i].size += size;
			return 0;	/* E}[ */
		}
	}
	if (man->frees < MEMMAN_FREES) {
		/* p^[3: OƂ܂Ƃ߂ꏊȂ̂ŐVɃXL} */
		/* free[i]Ԗڂɂ1ɂĂ炤 */
		for (j = man->frees; j < i; j--)
			man->free[j] = man->free[j - 1];
		man->frees++;
		if (man->maxfrees < man->frees)
			man->maxfrees = man->frees;	/* őlXV */
			man->free[i].addr = addr;
			man->free[i].size = size;
			return 0;	/* E}[ */
	}
	/* p^[4:ǂɂłȂ̂ŁuȂǎ̂ĂĂ܂v */
	man->losts++;
	man->lostsize += size;
	return -1;
}

unsigned int memman_alloc_4k(struct MEMMAN *man, unsigned int size)
{
	unsigned int a;
	size = (size + 0xfff) & 0xfffff000;
	a = memman_alloc(man, size);
	return a;
}

int memman_free_4k(struct MEMMAN *man, unsigned int addr, unsigned int size)
{
	unsigned int i;
	size = (size + 0xfff) & 0xfffff000;
	i = memman_free(man, addr, size);
	return i;
}

==> OSAkkie0.0.23.02.zip: src/memory.c <==
/* [ */

#include "bootpack.h"

#define EFLAGS_AC_BIT		0x00040000
#define CR0_CACHE_DISABLE	0x60000000

unsigned int memtest(unsigned int start, unsigned int end)
{
	char flg486 = 0;
	unsigned int eflg, cr0, i;

	/* 386A486ȍ~mF */
	eflg = io_load_eflags() | EFLAGS_AC_BIT;
	io_store_eflags(eflg);
	eflg = io_load_eflags() & EFLAGS_AC_BIT;
	if (eflg)
		flg486 = 1;
	eflg &= ~EFLAGS_AC_BIT;		// and̂
	io_store_eflags(eflg);

	if (flg486) {
		cr0 = load_cr0() | CR0_CACHE_DISABLE;	/* LbV֎~ */
		store_cr0(cr0);
	}

	i = memtest_sub(start, end);

	if (flg486) {
		cr0 = load_cr0();
		cr0 &= ~CR0_CACHE_DISABLE;	/* LbV */
		store_cr0(cr0);
	}

	return i;
}

void memman_init(struct MEMMAN *man)
{
	man->frees = 0;		/* 󂫏̌ */
	man->maxfrees = 0;	/* 󋵊ώ@p: frees̍ől */
	man->lostsize = 0;	/* JɎsvTCY */
	man->losts = 0;		/* JɎs */
	return;
}

unsigned int memman_total(struct MEMMAN *man)
/* 󂫃TCY̍v */
{
	unsigned int i, t = 0;
	for (i = 0; i < man->frees; i++)
		t += man->free[i].size;
	return t;
}

unsigned int memman_alloc(struct MEMMAN *man, unsigned int size)
/* m */
{
	unsigned int i, a;
	for (i = 0; i < man->frees; i++) {
		if (man->free[i].size >= size) {
			/* \ȍL̋󂫂𔭌 */
			a = man->free[i].addr;
			man->free[i].addr += size;
			man->free[i].size -= size;
			if (man->free[i].size == 0) {
				/* free[i]ȂȂ̂őOɋl߂ */
				man->frees--;
				for (; i < man->frees; i++)
					man->free[i] = man->free[i + 1];
			}
			return a;
		}
	}
	return 0;	/* 󂫂Ȃ */
}

int memman_free(struct MEMMAN *man, unsigned int addr, unsigned int size)
/*  */
{
	int i, j;

	/* free[]addrɕׂ邽߂ɁAǂɓ邩߂ */
	for (i = 0; i < man->frees; i++) {
		if (man->free[i].addr > addr)
			break;
	}
	/* free[i - 1].addr < addr < free[i].addr */
	if (i > 0) {
		/* p^[1-1: ꏊO */
		if (man->free[i - 1].addr + man->free[i - 1].size == addr) {
			/* ̑Ő󂫗̈ɂ܂Ƃ߂ */
			man->free[i - 1].size += size;
			if (i < man->frees) {
				/* p^[1-2: ꂽꏊ */
				if (addr + size == man->free[i].addr) {
					/* ̗̈܂Ƃ߂ */
					man->free[i - 1].size += man->free[i].size;
					/* man->free[i]̍폜:: Oɋl߂ */
					man->frees--;
					for (; i < man->frees; i++)
						man->free[i] = man->free[i + 1];
				}
			}
			return 0;	/* E}[ */
		}
	}
	if (i < man->frees) {
		/* p^[2: ꏊ낪 */
		if (addr + size == man->free[i].addr) {
			/* ̗̈ɂ܂Ƃ߂ */
			man->free[i].addr = addr;
			man->free[i].size += size;
			return 0;	/* E}[ */
		}
	}
	if (man->frees < MEMMAN_FREES) {
		/* p^[3: OƂ܂Ƃ߂ꏊȂ̂ŐVɃXL} */
		/* free[i]Ԗڂɂ1ɂĂ炤 */
		for (j = man->frees; j > i; j--)
			man->free[j] = man->free[j - 1];
		man->frees++;
		if (man->maxfrees < man->frees)
			man->maxfrees = man->frees;	/* őlXV */
			man->free[i].addr = addr;
			man->free[i].size = size;
			return 0;	/* E}[ */
	}
	/* p^[4:ǂɂłȂ̂ŁuȂǎ̂ĂĂ܂v */
	man->losts++;
	man->lostsize += size;
	return -1;
}

unsigned int memman_alloc_4k(struct MEMMAN *man, unsigned int size)
{
	unsigned int a;
	size = (size + 0xfff) & 0xfffff000;
	a = memman_alloc(man, size);
	return a;
}

int memman_free_4k(struct MEMMAN *man, unsigned int addr, unsigned int size)
{
	int i;
	size = (size + 0xfff) & 0xfffff000;
	i = memman_free(man, addr, size);
	return i;
}
